
package graphhandler;

/** Edge class that represents an edge in a graph
 */
public class Edge implements Comparable{
    /** source/target Nodes of the edge, and of course it's weight.*/
    private int source, target;
    private float weight;

    /** Constructor that builds an edge with the weight provided as the parameter. */
    Edge(float weight) {
        this.weight = weight;
    }

    /** Constructor that builds an edge based an the weight provided and by the source/target vertex parameters*/
    Edge(float weight, int source, int target){
        this.weight = weight;
        this.source = source;
        this.target = target;
    }

    /** Copy constructor. Builds an edge based on another edge */
    Edge(Edge e) {
        weight = e.weight;
        source = e.source;
        target = e.target;
    }

    /** Returns the weight of the edge. */
    public float getWeight() { return weight; }

    /** Sets the edge's Weight*/
    public void setWeight(float weight){ this.weight=weight;}
    
    /** Sets the source value in the edge. */
    public void setFrom(int source) { this.source = source; }

    /** Returns the source value of the edge. */
    public int getFrom() { return source; }

    /** Returns the target value of the edge. */
    public int getTo() { return target; }

    /** Sets the target value in the edge. */
    public void setTO(int target) { this.target = target; }


    /**Compares the two edges based on their weight value. */
    @Override public int compareTo(Object o) {
        if( ((Edge) o).getWeight() == weight   )
                return 0;
        else if( ((Edge) o).getWeight() < weight   )
                return -1;
        return 1;
    }

    @Override
    public int hashCode() {
        return 97 * (97 * (97 * 3 + this.source) + this.target) + Float.floatToIntBits(this.weight);
    }

    /** Compares the edge's weight with e's and returns true if they are equal */
    @Override public boolean equals(Object e) {
        if (e == null)  // Null references are not equal to this instance.
            return false;
        if (!e.getClass ().equals (Edge.class)) //Neither are they instances of different classes
            return false;
        return (((Edge) e).getTo() == target && ((Edge) e).getFrom() == source);
    }

    /** Returns the edge as a string. */
    @Override public String toString(){
        return "(" + source + ", " + target + ")";
    }

}
