
package graphhandler;

import java.io.File;
import javax.swing.filechooser.*;

/**  FileChooserFilter class is used to filter the files viewed by the FileChooser. */
public class FileChooserFilter extends FileFilter {

    /** The extensions the filter recognizes*/
    private final static String jpeg = "jpeg";
    private final static String jpg = "jpg";
    private final static String txt = "txt";
    private final static String png = "png";
    /** The description of the filter*/
    private String desc;
    private boolean showOnlyTxt;

    /** Default constructor*/
    public FileChooserFilter(){
        this(".txt files");
    }

    /** Constructor that takes as a parameter a Filter Description
     * @param description 
     */
    public FileChooserFilter(String description){
        if(description.equalsIgnoreCase("") || description==null){
            this.desc=".txt files";
            showOnlyTxt=true;
        } else{
            showOnlyTxt=false;
            this.desc=description;
        }
    }

    /** Accept all directories and all gif, jpg, tiff, or png files.
     * @param f 
     * @return 
     */
    @Override
    public boolean accept(File f) {
        if (f.isDirectory()) 
            return true;

        String extension = getExtension(f);
        if (extension != null) {
            if (extension.equals(txt) || 
                    ((extension.equals(jpeg) || extension.equals(jpg) || extension.equals(png)) && !showOnlyTxt) )
                    return true;
        }
        return false;
    }

    /** The description of this filter
     * @return 
     */
    @Override
    public String getDescription() {
        return desc;
    }

    /** Get the extension of a file.
     * @param file
     * @return  
     */
    public static String getExtension(File file) {
        String s = file.getName();
        int i = s.lastIndexOf('.');
        if (i > 0 &&  i < s.length() - 1) 
            return s.substring(i+1).toLowerCase();
        return null;
    }

    /** Check if the extension belongs to the ones recognised by the filter
     * @param extension
     * @return  
     */
    public static boolean isAcceptableExtension(String extension){
         if (extension.equalsIgnoreCase(txt) || extension.equalsIgnoreCase(jpeg)
                 || extension.equalsIgnoreCase(jpg) || extension.equalsIgnoreCase(png) )
                 return true;
         return false;
    }
}
