

package graphhandler;

import java.io.*;
import java.util.*;
import javax.swing.*;

public class GraphSCC {

    /** The graphs scc*/
    ArrayList <Graph> scc;

    /** Empty constructor. */
    public GraphSCC() {
        scc= new ArrayList<Graph>();
    }

    /** Constructs an ArrayList of graphs (scc) based on the original Graph*/
    public GraphSCC(Graph origin){
            findSCC(origin);
    }

    /** Returns the ArrayList containing the Strongly Connected (graph) Components*/
    public ArrayList<Graph> getSCCGraphs(){ return scc; }

    /** Returns the Graph contained in the 'position' given as argument*/
    public Graph get(int position){ return scc.get(position); }

    /**Prints the SCCs */
    public void printSCC(BufferedWriter stream) {
        try {
            for(int i=0; i<scc.size(); i++){
                scc.get(i).print(stream); stream.write("-----", 0, 4); }
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(new JFrame(), "Error, in printing Graph's SCC.", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    /**Finds the scc in the Graph */
    public ArrayList <Graph> findSCC(Graph g){
        //First get the transpose graph from the initial
        Graph GT = g.getTranspose();
        //Do a dfs to the transpose graph
        GT.DFS();
        //Do a descending sort of the graph's vertexs
        ArrayList Order = GT.sortByPost();
        //Do a DFS on G. The vertexs are searched based on the descending
        //DFSOrder sequence
        scc=g.orderedDFS(Order);
        return scc;
    }

    /**Returns number of scc's */
    public int size(){
        if(scc!= null)
            return scc.size();
        else return 0;
    }

    /**Returns the Edges between the SCC
     * @return returns a hashmap with the edges between the s.c.components
     */
    public HashMap<Integer, LinkedList<Integer>> getSCCEdges(Graph initialGraph){
        HashMap <Integer, Integer> nodeToScc= getNodeSCCMap();
        //Add the edges between the SCC by checking each time if the corresponding edge already exists (in sccEdges)
        HashMap <Integer, LinkedList <Integer>> sccEdges= new HashMap<Integer, LinkedList <Integer>>();
        for(int i=0; i<scc.size(); i++)
            sccEdges.put(i, new LinkedList<Integer>());

        Set verts = initialGraph.getVertexs().keySet();
        LinkedList list, targetsList; int id1, id2;
        for(Iterator iter= verts.iterator() ; iter.hasNext(); ){
            id1 =(Integer) iter.next();
            list= initialGraph.getAdjacencyMap().get(id1);
            targetsList= (LinkedList) sccEdges.get(nodeToScc.get(id1));
            for(int i=0;list!=null &&  i< list.size(); i++){
                id2 =((Edge) list.get(i)).getTo() ;
                if(targetsList!=null && !targetsList.contains(nodeToScc.get(id2)) && nodeToScc.get(id1)!=nodeToScc.get(id2))
                    targetsList.add(nodeToScc.get(id2));
            }
         }
        return sccEdges;
    }

    /**Returns the number of edges between the graph's scc
     * @return int showing the number of edges between the graph's scc
     */
    public int getSCCEdgeCount(Graph initialGraph) {
        int count = 0;
        HashMap<Integer, LinkedList<Integer>> edges=getSCCEdges(initialGraph);
        Set origins = edges.keySet();
        LinkedList keys;
        for(Iterator iter = origins.iterator(); iter.hasNext();){
                keys = edges.get((Integer) iter.next() );
                count += keys.size();
        }
        return count;
    }

    /**Returns a HashMap that corresponds the graph's nodes to the SCC they belong to
     * @return a hashmap that links a node to the scc it belongs to
     */
    private  HashMap <Integer, Integer> getNodeSCCMap( ){
        Set verts;
        HashMap <Integer, Integer> nodeToScc= new HashMap<Integer, Integer>();
        for(int i=0; i<scc.size(); i++){
            verts = scc.get(i).getVertexs().keySet();
            for(Iterator iter= verts.iterator() ; iter.hasNext(); ){
                nodeToScc.put((Integer) iter.next(), i);
            }
        }
        return nodeToScc;
    }

    /**Returns the biggest SCC. findSCC() must me called first.
    * @return returns null if findSCC() hasn't been called first.
    */
    public Graph getLargestSCC() {
        if (scc.isEmpty()) {
            return null;
        }
        int pos = 0,max = -1;
        for (int i = 0; i < scc.size(); i++) {
            if (scc.get(i).getNodesNum() > max) {
                max = scc.get(i).getNodesNum();
                pos = i;
            }
        }
        return scc.get(pos);
    }

}
 