
/**TO DO's:
 1) Fix various Memory Leaks (eg. in tabs)
 2) Make Vertex, Edge, UNode work with Generic
 */
package graphhandler;

import graphComponents.Graph;
import java.io.*;
import java.util.*;
import javax.swing.UIDefaults;
import javax.swing.UIManager;

/** Main class that starts the gui*/
public class Main {

   public static void main2(String[] args) throws IOException{
        Scanner in= new Scanner(System.in);
        System.out.println("1) Run Kruskal for the default number of nodes/all graph types\n2) Run Kruskal for custom nodes/types\n" +
                            "3) Run Kruskal for custom nodes for all graph types\n4) Exit");
        String choice=in.next();
        while(!choice.equalsIgnoreCase("4")){
            if(choice.equalsIgnoreCase("1")){
                runDefaultTest(0);
            }else if(choice.equalsIgnoreCase("2")){
                try{
                    System.out.println("Please enter the number of nodes you want \nthe random graph to have, followed by it's type(1,2,3):");
                    Graph temp=makeGraph(in.nextInt(), in.nextInt());
                    System.out.println("First case: "+ temp.Kruskal().getGraphsWeight());
                    temp=null;System.gc();
                }catch(Exception e){ System.out.println("Wrong Input! Only integers please.");}
            }else if(choice.equalsIgnoreCase("3")){
                try{
                    System.out.println("Please enter the number of nodes you want the random graph to have:");
                    runDefaultTest(in.nextInt());
                }catch(Exception e){ System.out.println("Wrong Input! Only integers please.");}
            }else break;
            
            System.out.println("1) Run Kruskal for the default number of nodes/all graph types\n2) Run Kruskal for custom numbers\n3) Exit");
        }    
    }

   /** Runs the kruskal algorithm for a given set of nodes or for the default if nodes==0*/
   public static void runDefaultTest(int nodes) throws IOException{
        if(nodes==0){
            int[] nodeNums={10, 100, 200, 500, 1000, 2000, 5000, 6000, 7000};
            for(int i=0; i< nodeNums.length; i++){
                System.out.println("For n= "+nodeNums[i]+" the MST weight is:");
                Graph temp=makeGraph(1, nodeNums[i]);
                System.out.println("First case: "+ temp.Kruskal().getGraphsWeight());
                temp=null;System.gc();
                temp=makeGraph(2, nodeNums[i]);
                System.out.println("Second case: "+ temp.Kruskal().getGraphsWeight());
                temp=null;System.gc();
                temp=makeGraph(3, nodeNums[i]);
                System.out.println("Third case: "+ temp.Kruskal().getGraphsWeight());
                temp=null;System.gc();
            }
        }else if(nodes>0){
                System.out.println("For n= "+nodes+" the MST weight is:");
                Graph temp=makeGraph(1, nodes);
                System.out.println("First case: "+ temp.Kruskal().getGraphsWeight());
                temp=null;System.gc();
                temp=makeGraph(2, nodes);
                System.out.println("Second case: "+ temp.Kruskal().getGraphsWeight());
                temp=null;System.gc();
                temp=makeGraph(3, nodes);
                System.out.println("Third case: "+ temp.Kruskal().getGraphsWeight());
                temp=null;System.gc();
        }

   }

   /** Makes a Graph with the edges for the corresponding types/nodes*/
   public static Graph makeGraph(int type, int nodesNum) throws IOException{
        Graph target=new Graph();
        Random rmaker=new Random();
        //Generate the Edges according to the type
        for(short i=0; i<nodesNum; i++){
            for(short j=i; j<nodesNum; j++){
                if(i!=j){
                    if(type==1){
                        target.addEdge( i, j , rmaker.nextFloat());
                    }else if(type==2){
                        Float distance= ((Double) Math.sqrt(Math.pow(rmaker.nextFloat()-rmaker.nextFloat(), 2d) +
                                                  Math.pow(rmaker.nextFloat()-rmaker.nextFloat(), 2d) )).floatValue();
                        target.addEdge( i , j , distance);
                    }else {
                        Float x1,y1,x2,y2;
                        while( Math.pow((x1=rmaker.nextFloat()), 2d) +  Math.pow((y1=rmaker.nextFloat()), 2d) >1 );
                        while( Math.pow((x2=rmaker.nextFloat()), 2d) +  Math.pow((y2=rmaker.nextFloat()), 2d) >1 );
                        Float distance=((Double) Math.sqrt(Math.pow(x2-x1, 2d) + Math.pow(y2-y1, 2d))).floatValue() ;
                        target.addEdge( i , j , distance);
                    }
                }
            }
        }
        System.gc();
        return target;
    }

   /** Makes a Graph file with the edges for the corresponding types/nodes*/
   public static File makeGraphFile(String name, int type, int nodesNum) throws IOException{
        File target=new File(name);
        //if target doesn't exist create it
        if(!target.exists())
            target.createNewFile();
        Random rmaker=new Random();
        BufferedWriter output = new BufferedWriter(new FileWriter(target));
        //Generate the Edges according to the type
        for(int i=0; i<nodesNum; i++){
            for(int j=i; j<nodesNum; j++){
                if(i!=j){
                    if(type==1){
                        output.write( i + "\t" + j +"\t"+ rmaker.nextDouble() +"\n");
                    }else if(type==2){
                        Double distance=Math.sqrt(Math.pow(rmaker.nextDouble()-rmaker.nextDouble(), 2d) +
                                                  Math.pow(rmaker.nextDouble()-rmaker.nextDouble(), 2d) );
                        output.write( i + "\t" + j +"\t"+ distance +"\n");
                    }else {
                        Double x1,y1,x2,y2;
                        while( Math.pow((x1=rmaker.nextDouble()), 2d) +  Math.pow((y1=rmaker.nextDouble()), 2d) >1 );
                        while( Math.pow((x2=rmaker.nextDouble()), 2d) +  Math.pow((y2=rmaker.nextDouble()), 2d) >1 );
                        Double distance=Math.sqrt(Math.pow(x2-x1, 2d) + Math.pow(y2-y1, 2d) );
                        output.write( i + "\t" + j +"\t"+ distance +"\n");
                    }
                }
            }
        }
        //End output
        output.flush();
        output.close();
        System.gc();
        return target;
    }

   public static void main(String[] args) throws IOException {
        //makeGraphFile("weighted.txt", 1, 50);
        try {
            System.setProperty("awt.useSystemAAFontSettings", "on");
            System.setProperty("swing.aatext", String.valueOf(true));
            boolean lookAndFeel = false;
            for (UIManager.LookAndFeelInfo laf : UIManager.getInstalledLookAndFeels()) {
                if (laf.getName().equalsIgnoreCase("GTK+") || laf.getName().equalsIgnoreCase("Windows Vista")) {
                    UIManager.setLookAndFeel(laf.getClassName());
                    lookAndFeel = true;
                    break;
                }
            }
            UIDefaults uiDefaults = UIManager.getDefaults();
            uiDefaults.put("TextArea.font", uiDefaults.get("TextPane.font"));
            if (!lookAndFeel) {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            }
        }catch(Exception e){}
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                GraphVisualizer visualG = new GraphVisualizer();
            }
        });

    }

}
