

package graphhandler;

/** Vertex class that represents a node in a graph
 */
public class Vertex implements Comparable{
    /** Post, pre values of the node and it's id */
    private int post, pre;
    private int id;

    /** Constructor that builds a vertex with the id provided as the parameter. */
    Vertex(int id) {
        this.id = id;
    }
    
    /** Copy constructor. Builds a vertex based on another vertex */
    Vertex(Vertex v) {
        this.id = v.id;
        this.post = v.post;
        this.pre = v.pre;
    }

    /** Returns the id of the vertex. */
    public int getId() { return this.id; }

    /** Sets the post value in the vertex. */
    public void setPost(int post) { this.post = post; }

    /** Returns the post value of the vertex. */
    public int getPost() { return this.post; }

    /** Returns the pre value of the vertex. */
    public int getPre() { return this.pre; }

    /** Sets the pre value in the vertex. */
    public void setPre(int pre) { this.pre = pre; }

    /**Compares the two Vertexs based on their post value. */
    @Override public int compareTo(Object o) {
        if( ((Vertex) o).getPost() == this.post   )
                return 0;
        else if( ((Vertex) o).getPost() < this.post   )
                return -1;
        return 1;
    }

    @Override public int hashCode() {
        return 47 * 5 + this.id;
    }

    /** Compares the vertex's id with v's and returns true if they are equal */
    @Override public boolean equals(Object v) {
        if (v == null)  // Null references are not equal to this instance.
            return false;
        if (!v.getClass ().equals (Vertex.class)) //Neither are they instances of different classes
            return false;
        return (((Vertex) v).id == this.id);
    }



    /** Returns the vertex's id as a string. */
    @Override public String toString(){
        return "" + this.id;
    }
    
}
